use clap::{ArgAction, Arg, Command};
use regex::Regex;
use std::process::Command as Cmd;
use std::str;

const IPMIHOST: &str = "192.168.1.223";
const IPMIUSER: &str = "root";
const IPMIPASS: &str = "<password>";

fn execute_ipmi_command(command: &str) -> String {
    let full_command = format!(
        "ipmitool -I lanplus -H {} -U {} -P '{}' {}",
        IPMIHOST, IPMIUSER, IPMIPASS, command
    );

    let output = Cmd::new("sh")
        .arg("-c")
        .arg(full_command)
        .output()
        .expect("Failed to execute command");

    let stdout = str::from_utf8(&output.stdout).unwrap_or("");
    let stderr = str::from_utf8(&output.stderr).unwrap_or("");
    
    if !stderr.is_empty() {
        eprintln!("{}", stderr);
    }

    stdout.trim().to_string()
}

fn power_on() {
    println!("Powering on the server...");
    execute_ipmi_command("chassis power on");
}

fn power_off() {
    println!("Powering off the server...");
    execute_ipmi_command("chassis power off");
    println!("Enabling dynamic fan control...");
    execute_ipmi_command("raw 0x30 0x30 0x01 0x01");
}

fn set_fan_speed(speed: u8) {
    println!("Setting fan speed to {}%...", speed);
    execute_ipmi_command(&format!("raw 0x30 0x30 0x02 0xff {:x}", speed));
}

fn enable_dynamic_fan_control() {
    println!("Enabling dynamic fan control...");
    execute_ipmi_command("raw 0x30 0x30 0x01 0x01");
}

fn disable_dynamic_fan_control() {
    println!("Disabling dynamic fan control...");
    execute_ipmi_command("raw 0x30 0x30 0x01 0x00");
}

fn get_server_temperature() -> f32 {
    println!("Getting server temperature...");
    let command = format!(
        "ipmitool -I lanplus -H {} -U {} -P '{}' sdr type temperature",
        IPMIHOST, IPMIUSER, IPMIPASS
    );
    let result = Cmd::new("sh")
        .arg("-c")
        .arg(command)
        .output()
        .expect("Failed to get temperature");

    let stdout = str::from_utf8(&result.stdout).unwrap_or("");

    let re = Regex::new(r"\| (\d+) degrees C").unwrap();
    let mut temps = Vec::new();

    for cap in re.captures_iter(stdout) {
        if let Ok(temp) = cap[1].parse::<f32>() {
            temps.push(temp);
        }
    }

    let avg_temp = if !temps.is_empty() {
        temps.iter().sum::<f32>() / temps.len() as f32
    } else {
        0.0
    };
    
    avg_temp
}

fn adjust_fan_speed_based_on_temp() {
    let temp = get_server_temperature();
    println!("Current temperature: {:.1} degrees C", temp);

    let warning_threshold = 75.0;
    let critical_threshold = 90.0;

    if temp >= critical_threshold {
        println!("Temperature is critical, setting fan speed to 100%.");
        set_fan_speed(100);
    } else if temp >= warning_threshold {
        println!("Temperature is high, setting fan speed to 80%.");
        set_fan_speed(80);
    } else if temp >= 40.0 {
        println!("Temperature is moderate, setting fan speed to 60%.");
        set_fan_speed(60);
    } else if temp >= 30.0 {
        println!("Temperature is low, setting fan speed to 50%.");
        set_fan_speed(50);
    } else {
        println!("Temperature is very low, setting fan speed to 40%.");
        set_fan_speed(40);
    }
}

fn main() {
    let matches = Command::new("IDRAC helper script.")
        .version("1.0")
        .about("Controls server power, fan speed, and temperature monitoring for Dell servers")
        .arg(
            Arg::new("power")
                .short('p')
                .long("power")
                .value_parser(["on", "off"])
                .help("Power on or off the server"),
        )
        .arg(
            Arg::new("fan")
                .short('f')
                .long("fan")
                .value_parser(clap::value_parser!(u8).range(0..=100))
                .help("Set fan speed percentage"),
        )
        .arg(
            Arg::new("temp")
                .short('t')
                .long("temp")
                .action(ArgAction::SetTrue)
                .help("Adjust fan speed based on temperature"),
        )
        .arg(
            Arg::new("dynamic")
                .short('d')
                .long("dynamic")
                .value_parser(["on", "off"])
                .help("Toggle dynamic fan control"),
        )
        .get_matches();

    if let Some(power) = matches.get_one::<String>("power") {
        match power.as_str() {
            "on" => power_on(),
            "off" => power_off(),
            _ => {}
        }
    }

    if let Some(fan_speed) = matches.get_one::<u8>("fan") {
        set_fan_speed(*fan_speed);
    }

    if matches.get_flag("temp") {
        adjust_fan_speed_based_on_temp();
    }

    if let Some(dynamic) = matches.get_one::<String>("dynamic") {
        match dynamic.as_str() {
            "on" => enable_dynamic_fan_control(),
            "off" => disable_dynamic_fan_control(),
            _ => {}
        }
    }
}
